import { getPublicKey, loginRequest } from '@/api/login';
import { TOKEN_CACHE_NAME, loginInfo } from '@/config/setting';
import { LoginResult } from '@/types/Login';
import { pwdEncryption } from './jsencrypt';

type Info = {
  userInfo: Partial<LoginResult>;
  appId: string;
};

type AppInfo = {
  otherWebJson: null | string;
  path: string;
  self: string;
  name: string;
};

/** 获取当前用户信息,包含 appId 和 userInfo */
export const getUserInfo = () => {
  const info = sessionStorage.getItem(TOKEN_CACHE_NAME);
  return info ? (JSON.parse(info) as Info) : null;
};

/**
 * 获取缓存的token
 */
export function getToken(): string | null {
  const token = localStorage.getItem(TOKEN_CACHE_NAME);
  if (!token) {
    return sessionStorage.getItem(TOKEN_CACHE_NAME);
  }
  return token;
}

/** 获取点击的应用的信息 */
export const getAppInfo = () => {
  const info = sessionStorage.getItem('usePortalDepartment');
  return info ? (JSON.parse(info).appConfig as AppInfo) : null;
};

/** 本地开发登录,设置用户信息 */
export const mockLogin = async (appId: string) => {
  const pwdKeyRes = await getPublicKey();
  const params = loginInfo;
  params.password = pwdEncryption(pwdKeyRes.data, params.password) as any;
  const { data } = await loginRequest(params);
  setUserInfo(
    JSON.stringify({
      appId,
      userInfo: data
    })
  );
};
/** 设置用户信息，只在本地开发使用 */
export const setUserInfo = (val: string) => {
  sessionStorage.setItem(TOKEN_CACHE_NAME, val);
};
/**
 * 移除token
 */
export function removeToken() {
  sessionStorage.removeItem(TOKEN_CACHE_NAME);
  sessionStorage.removeItem('useTabsStore');
  sessionStorage.removeItem('useWebSocket');
  sessionStorage.removeItem('usePortalDepartment');
}
