import type { ComputedRef } from 'vue';
import { computed } from 'vue';
import { storeToRefs } from 'pinia';
import { useDictionary } from '@/store/modules/dictionary';
import type { TDictionaryData } from '@/types/DictionaryStore';
import { getDictDetailDataRequest } from '@/api/common/dictionary';
/**
 * 获取字典数据hook
 * @param codes 字典编码
 */
export function useDictData(codes: string[]): ComputedRef<TDictionaryData[]>[] {
  const result: ComputedRef<TDictionaryData[]>[] = [];

  // 已缓存的字典
  const useDict = useDictionary();
  const { dicts } = storeToRefs(useDict);

  codes.forEach(async (code) => {
    if (!code) return; // 检查 code 是否为空或无效
    result.push(computed<TDictionaryData[]>(() => dicts.value[code] || []));
    // 若还未缓存过则获取字典数据
    if (dicts.value[code] != null) {
      return;
    }
    useDict.setDicts([], code);

    const params = {
      pageNo: 1,
      pageSize: 1000,
      typeCode: code
    };

    const { code: retCode, data } = await getDictDetailDataRequest(params);
    if (retCode !== 200) return;

    const ret = data.records;

    ret.map((item) => {
      item.dictDataCode = item.dictCode;
      item.dictDataName = item.dictName;
    });

    useDict.setDicts(data.records, code);
  });

  return result;
}
